/*
 * plpbtrom.c - generate pci boot manager rom 
 *
 * Released under GPL v2
 * by Elmar Hanlhofer  http://www.plop.at
 *
 */


#define VERSION "0.6 20110814"




#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>

#include "lzarilib.h"

#ifdef WIN32
#include "windows.h"
#include "winioctl.h"
#endif

#include <stdlib.h>
#include <string.h>

#define ISA 0
#define PCI 1



void printHelp()
{

    printf ("plpbtrom " VERSION " by Elmar Hanlhofer http://www.plop.at\n\n");
    printf ("Create a pci option rom with the plop boot manager.\n\n");
    printf ("Usage: plpbtrom [OPTIONS] input-file output-file\n\n");
    printf ("Options:\n");
    printf ("========\n");

    printf (" -h               this help\n");
    printf (" -vendorid        id hex value of the card vendor\n");
    printf (" -deviceid        id hex value of device\n");
    printf (" -grabid          grab vendor/device id from PCI ROM file\n");
    printf (" -hidden          disable option rom text output\n");
    printf (" -nodisable       do not allow option rom disable\n");
    printf (" -forceINT        force hook interrupt\n");
    printf (" -INT18           hook int 18h (see www.plop.at for details)\n");
    printf (" -compress        compress boot manager for a smaller  rom\n");
    printf (" -size <val[k|m]> set rom to a size. examples: 64k, 128k, 1m, ...\n");
    printf ("\n");
    printf (" input-file   boot manager file (ex. plpbtrom.bin)\n");
    printf (" output-file  ROM file name\n");

    printf ("\n");
    printf ("Examples:\n");
    printf ("=========\n");
    printf (" plpbtrom plpbtrom.bin plpbt.rom\n");
    printf (" plpbtrom -vendorid 0x10ec -deviceid 0x8139 plpbtrom.bin plpbt.rom\n");
    printf (" plpbtrom -grabid netcard.rom plpbtrom.bin plpbt.rom\n");
    printf (" plpbtrom -grabid netcard.rom -size 64k plpbtrom.bin plpbt.rom\n");
    printf ("\n");

}

int param (char *t, char *val, char *arg)
{
    if (strncmp (arg, t, strlen (t)) == 0) 
    {
	strcpy (val, arg + strlen(t));
	return 1;
    }

    return 0;
}

int tohex (char *t)
{
    int i = 0;

    if ((t[0] >= 'a') && (t[0] <= 'f')) i = t[0] - 'a' + 10;
    if ((t[0] >= 'A') && (t[0] <= 'F')) i = t[0] - 'A' + 10;
    if ((t[0] >= '0') && (t[0] <= '9')) i = t[0] - '0';
    if (t[1] == 0x00) return (i);


    i *= 16;
    if ((t[1] >= 'a') && (t[1] <= 'f')) i += t[1] - 'a' + 10;
    if ((t[1] >= 'A') && (t[1] <= 'F')) i += t[1] - 'A' + 10;
    if ((t[1] >= '0') && (t[1] <= '9')) i += t[1] - '0';
    if (t[2] == 0x00) return (i);


    i *= 16;
    if ((t[2] >= 'a') && (t[2] <= 'f')) i += t[2] - 'a' + 10;
    if ((t[2] >= 'A') && (t[2] <= 'F')) i += t[2] - 'A' + 10;
    if ((t[2] >= '0') && (t[2] <= '9')) i += t[2] - '0';
    if (t[3] == 0x00) return (i);

    i *= 16;
    if ((t[3] >= 'a') && (t[3] <= 'f')) i += t[3] - 'a' + 10;
    if ((t[3] >= 'A') && (t[3] <= 'F')) i += t[3] - 'A' + 10;
    if ((t[3] >= '0') && (t[3] <= '9')) i += t[3] - '0';

    return (i);

}


//========================================================

int main (int argc, char *argv[])
{
    unsigned int size = 64 * 1024;


    char inputfile[1024];
    char outputfile[1024];
    unsigned int filesize;
    FILE *pf;
    unsigned int fullsize;
    unsigned char chksum = 0;
    unsigned int i;
    int i2;
    int vendorid = 0;
    int deviceid = 0;
    int hidden = 0;
    int disable = 1;
    int forceINT = 0;
    int INT18 = 0;
    int encode = 0;
    unsigned int cfilesize = 0;
    int paramok;
    int setsize = 0;
    

		
    char pciheader[] = {
		0x55,0xaa,0x56,0xe9,0xaf,0x00,0xcb,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x1c,0x00,0x34,0x00,0x50,0x43,0x49,0x52,0xec,
		0x10,0x39,0x81,0x00,0x00,0x18,0x00,0x00,0x02,0x00,0x00,
		0x56,0x00,0x00,0x00,0x00,0x80,0x00,0x00,0x24,0x50,0x6e,
		0x50,0x01,0x02,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,
		0x59,0x00,0x7c,0x00,0x02,0x00,0x00,0x64,0x00,0x00,0x00,
		0x00,0x8e,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00,
		0x01,0x50,0x6c,0x6f,0x70,0x20,0x2d,0x20,0x45,0x6c,0x6d,
		0x61,0x72,0x20,0x48,0x61,0x6e,0x6c,0x68,0x6f,0x66,0x65,
		0x72,0x20,0x77,0x77,0x77,0x2e,0x70,0x6c,0x6f,0x70,0x2e,
		0x61,0x74,0x00,0x50,0x6c,0x6f,0x70,0x20,0x42,0x6f,0x6f,
		0x74,0x20,0x4d,0x61,0x6e,0x61,0x67,0x65,0x72,0x00,0x2e,
		0x80,0x3e,0xcd,0x02,0x01,0x74,0x1e,0x9c,0x60,0x1e,0xfc,
		0xbe,0xce,0x02,0x0e,0x1f,0xbf,0x00,0x01,0xb8,0x00,0x20,
		0x8e,0xc0,0xb9,0x50,0xc3,0xf3,0xa4,0x9a,0x00,0x01,0x00,
		0x20,0x1f,0x61,0x9d,0xcb,0x9c,0x1e,0x06,0x66,0x60,0x0e,
		0x1f,0xfc,0x80,0x3e,0x58,0x00,0x00,0x74,0x05,0xe8,0xb3,
		0x00,0x72,0x47,0x80,0x3e,0x55,0x00,0x01,0x74,0x10,0xe8,
		0x46,0x00,0x72,0x0b,0xe8,0x31,0x01,0xbe,0xaa,0x02,0xe8,
		0x0b,0x01,0xeb,0x30,0xbb,0x66,0x01,0xbf,0x64,0x00,0xb1,
		0x39,0x80,0x3e,0x56,0x00,0x00,0x74,0x05,0xbf,0x60,0x00,
		0xb1,0x38,0xe8,0x59,0x00,0x66,0x89,0x1e,0x74,0x01,0xe8,
		0x0a,0x01,0xbe,0xbc,0x02,0xe8,0xe4,0x00,0x8a,0xc1,0xe8,
		0xce,0x00,0xbe,0xc8,0x02,0xe8,0xd9,0x00,0x66,0x61,0x07,
		0x1f,0x9d,0xb8,0x01,0x00,0xcb,0xbb,0xf0,0xef,0x83,0xc3,
		0x01,0x74,0x10,0x8e,0xc3,0x66,0x26,0x81,0x3e,0x00,0x00,
		0x24,0x50,0x6e,0x50,0x75,0xed,0xf8,0xc3,0xf9,0xc3,0x52,
		0x50,0xfb,0xf4,0xba,0x70,0x00,0xb0,0x0a,0xee,0x42,0xec,
		0xa8,0x80,0x75,0xf2,0x58,0x66,0x83,0xe0,0x0f,0xba,0x70,
		0x00,0xee,0x42,0xec,0x5a,0xc3,0x06,0x6a,0x00,0x07,0x8c,
		0xc8,0x66,0xc1,0xe0,0x10,0x8b,0xc3,0x66,0x26,0x8b,0x1d,
		0x66,0x26,0x89,0x05,0x07,0xc3,0x0e,0xe8,0x2c,0xff,0x66,
		0x2e,0x83,0x3e,0x74,0x01,0x00,0x74,0x05,0xea,0x00,0x00,
		0x00,0x00,0xcd,0x18,0xb0,0x00,0xe8,0xb4,0xff,0x8a,0xd8,
		0xb9,0x03,0x00,0xbe,0x4d,0x02,0xe8,0x8c,0x00,0x8a,0xc1,
		0x04,0x30,0xe8,0x47,0x00,0xbe,0x73,0x02,0xe8,0x52,0x00,
		0x60,0xb4,0x01,0xcd,0x16,0x74,0x27,0x0c,0x60,0x3c,0x64,
		0x75,0x15,0x33,0xc0,0xcd,0x16,0xc6,0x06,0xcd,0x02,0x01,
		0xe8,0x44,0x00,0xbe,0x7b,0x02,0xe8,0x60,0x00,0x61,0xf9,
		0xc3,0xe8,0x38,0x00,0xbe,0x95,0x02,0xe8,0x54,0x00,0x61,
		0xf8,0xc3,0x61,0xb0,0x00,0xe8,0x68,0xff,0x3a,0xd8,0x74,
		0xc8,0x8a,0xd8,0xe2,0xb1,0xe8,0x1e,0x00,0xf8,0xc3,0x80,
		0x3e,0x57,0x00,0x01,0x74,0x09,0x60,0xb4,0x0e,0xbb,0x07,
		0x00,0xcd,0x10,0x61,0xc3,0xac,0x0a,0xc0,0x74,0x05,0xe8,
		0xe7,0xff,0xeb,0xf6,0xc3,0xb0,0x0d,0xe8,0xdf,0xff,0xb9,
		0x4f,0x00,0xb0,0x20,0xe8,0xd7,0xff,0xe2,0xfb,0xb0,0x0d,
		0xe8,0xd0,0xff,0xc3,0xbe,0x7c,0x00,0xe8,0xda,0xff,0xbe,
		0x3e,0x02,0xe8,0xd4,0xff,0xc3,0x56,0xbe,0xa8,0x02,0xe8,
		0xcc,0xff,0xbe,0x7c,0x00,0xe8,0xc6,0xff,0xbe,0x2e,0x02,
		0xe8,0xc0,0xff,0x5e,0xe8,0xbc,0xff,0xc3,0x20,0x2d,0x20,
		0x4f,0x70,0x74,0x69,0x6f,0x6e,0x20,0x52,0x4f,0x4d,0x3a,
		0x20,0x00,0x20,0x69,0x6e,0x69,0x74,0x69,0x61,0x6c,0x69,
		0x7a,0x65,0x64,0x2c,0x20,0x00,0x70,0x72,0x65,0x73,0x73,
		0x20,0x44,0x20,0x74,0x6f,0x20,0x64,0x69,0x73,0x61,0x62,
		0x6c,0x65,0x20,0x74,0x68,0x65,0x20,0x62,0x6f,0x6f,0x74,
		0x20,0x6d,0x61,0x6e,0x61,0x67,0x65,0x72,0x2e,0x20,0x00,
		0x20,0x73,0x65,0x63,0x2e,0x2e,0x2e,0x00,0x62,0x6f,0x6f,
		0x74,0x20,0x6d,0x61,0x6e,0x61,0x67,0x65,0x72,0x20,0x64,
		0x69,0x73,0x61,0x62,0x6c,0x65,0x64,0x2e,0x0d,0x0a,0x0a,
		0x00,0x64,0x69,0x73,0x61,0x62,0x6c,0x65,0x20,0x73,0x6b,
		0x69,0x70,0x70,0x65,0x64,0x2e,0x0d,0x0a,0x00,0x0d,0x00,
		0x75,0x73,0x69,0x6e,0x67,0x20,0x50,0x4e,0x50,0x20,0x6d,
		0x6f,0x64,0x65,0x0d,0x0a,0x0a,0x00,0x68,0x6f,0x6f,0x6b,
		0x65,0x64,0x20,0x49,0x4e,0x54,0x31,0x00,0x68,0x0d,0x0a,
		0x0a,0x00,0x00
	    };

    char decoder[] = {
		0x8c,0xc8,0x8e,0xd8,0x8e,0xc0,0x8e,0xd0,0xbc,0xfe,0xff,
		0x52,0xbf,0x50,0xc3,0xbe,0x2c,0x89,0xfd,0xa4,0x81,0xff,
		0x9f,0x3f,0x75,0xf9,0xfc,0xbf,0x7c,0x05,0xb9,0x24,0x3a,
		0x32,0xc0,0xf3,0xaa,0xbe,0xa0,0x3f,0x8e,0x06,0x65,0x05,
		0xc7,0x06,0x7c,0x05,0x00,0x01,0xe8,0xb0,0x03,0x5a,0x8e,
		0x06,0x65,0x05,0x06,0xbf,0x00,0x01,0x57,0xcb,0x60,0xd0,
		0x2e,0x69,0x05,0x75,0x12,0x8b,0x36,0x7e,0x05,0x8a,0x04,
		0xa2,0x6a,0x05,0xff,0x06,0x7e,0x05,0xc6,0x06,0x69,0x05,
		0x80,0xa0,0x6a,0x05,0x22,0x06,0x69,0x05,0x3c,0x00,0x74,
		0x02,0xb0,0x01,0x32,0xe4,0xa3,0x9e,0x3f,0x61,0xc3,0x60,
		0xc7,0x06,0x8f,0x1f,0x00,0x00,0xbe,0x3a,0x01,0x8b,0xfe,
		0x4f,0xd1,0xe7,0x89,0xb5,0xbb,0x15,0xd1,0xef,0xd1,0xe6,
		0x89,0xbc,0x2f,0x18,0xc7,0x84,0xa5,0x1a,0x01,0x00,0x8b,
		0x84,0x1b,0x1d,0x03,0x84,0xa5,0x1a,0x89,0x84,0x19,0x1d,
		0xd1,0xee,0x4e,0x83,0xfe,0x00,0x77,0xd5,0xc7,0x06,0xa5,
		0x1a,0x00,0x00,0xc7,0x06,0x91,0x3f,0x00,0x00,0xb9,0x00,
		0x10,0xbe,0x00,0x20,0x8b,0xd9,0x81,0xc3,0xc8,0x00,0x33,
		0xd2,0xb8,0x10,0x27,0xf7,0xf3,0x03,0x84,0x91,0x1f,0x89,
		0x84,0x8f,0x1f,0x49,0x83,0xee,0x02,0x83,0xf9,0x00,0x77,
		0xe2,0x61,0xc3,0x60,0x8b,0xec,0x83,0xc5,0x12,0x81,0x3e,
		0x1b,0x1d,0xff,0x7f,0x72,0x1c,0x33,0xdb,0xb9,0x3a,0x01,
		0x8b,0xf1,0xd1,0xe6,0x89,0x9c,0x1b,0x1d,0x8b,0x84,0xa5,
		0x1a,0x40,0xd1,0xe8,0x89,0x84,0xa5,0x1a,0x03,0xd8,0xe2,
		0xe9,0x8b,0x4e,0x00,0x8b,0xf1,0xd1,0xe6,0x8b,0x84,0xa5,
		0x1a,0x3b,0x84,0xa3,0x1a,0x75,0x03,0x49,0xeb,0xef,0x3b,
		0x4e,0x00,0x73,0x2c,0x8b,0xf1,0xd1,0xe6,0x8b,0x7e,0x00,
		0xd1,0xe7,0x8b,0x94,0x2f,0x18,0x8b,0x9d,0x2f,0x18,0x89,
		0x9c,0x2f,0x18,0x89,0x95,0x2f,0x18,0x53,0xd1,0xe2,0x8b,
		0xda,0xd1,0xef,0x89,0xbf,0xbb,0x15,0x5b,0xd1,0xe3,0x51,
		0x8f,0x87,0xbb,0x15,0x8b,0xf9,0xd1,0xe7,0xff,0x85,0xa5,
		0x1a,0x49,0x78,0x0a,0x8b,0xf1,0xd1,0xe6,0xff,0x84,0x1b,
		0x1d,0xeb,0xf3,0x61,0xc3,0x60,0x8b,0xec,0x8b,0x56,0x12,
		0xb9,0x01,0x00,0xbb,0x3a,0x01,0x3b,0xcb,0x73,0x19,0x8b,
		0xf3,0x03,0xf1,0xd1,0xee,0x8b,0xfe,0xd1,0xe6,0x39,0x94,
		0x1b,0x1d,0x76,0x05,0x8b,0xcf,0x41,0xeb,0x02,0x8b,0xdf,
		0xeb,0xe3,0x89,0x0e,0x9e,0x3f,0x61,0xc3,0x60,0x8b,0xec,
		0x8b,0x56,0x12,0xb9,0x01,0x00,0xbb,0x00,0x10,0x3b,0xcb,
		0x73,0x19,0x8b,0xf3,0x03,0xf1,0xd1,0xee,0x8b,0xfe,0xd1,
		0xe6,0x39,0x94,0x91,0x1f,0x76,0x05,0x8b,0xcf,0x41,0xeb,
		0x02,0x8b,0xdf,0xeb,0xe3,0x49,0x89,0x0e,0x9e,0x3f,0x61,
		0xc3,0x66,0x60,0xb9,0x11,0x00,0x66,0x8b,0x1e,0x74,0x05,
		0x66,0xd1,0xe3,0xe8,0x77,0xfe,0x66,0x0f,0xb7,0x06,0x9e,
		0x3f,0x66,0x03,0xd8,0x66,0x89,0x1e,0x74,0x05,0xe2,0xe5,
		0x66,0x61,0xc3,0x66,0x60,0x66,0x8b,0x1e,0x70,0x05,0x66,
		0x2b,0x1e,0x6c,0x05,0x66,0xa1,0x74,0x05,0x66,0x2b,0x06,
		0x6c,0x05,0x66,0x40,0x66,0x0f,0xb7,0x0e,0x1b,0x1d,0x66,
		0xf7,0xe1,0x66,0x48,0x66,0x33,0xd2,0x66,0xf7,0xf3,0x50,
		0xe8,0x52,0xff,0x58,0x8b,0x36,0x9e,0x3f,0xd1,0xe6,0x66,
		0x0f,0xb7,0x84,0x19,0x1d,0x66,0xf7,0xe3,0x66,0x33,0xd2,
		0x66,0x0f,0xb7,0x0e,0x1b,0x1d,0x66,0xf7,0xf1,0x66,0x03,
		0x06,0x6c,0x05,0x66,0xa3,0x70,0x05,0x66,0x0f,0xb7,0x84,
		0x1b,0x1d,0x66,0xf7,0xe3,0x66,0x33,0xd2,0x66,0xf7,0xf1,
		0x66,0x01,0x06,0x6c,0x05,0x66,0x81,0x3e,0x6c,0x05,0x00,
		0x00,0x01,0x00,0x72,0x1d,0x66,0x81,0x2e,0x74,0x05,0x00,
		0x00,0x01,0x00,0x66,0x81,0x2e,0x6c,0x05,0x00,0x00,0x01,
		0x00,0x66,0x81,0x2e,0x70,0x05,0x00,0x00,0x01,0x00,0xeb,
		0x3e,0x66,0x81,0x3e,0x6c,0x05,0x00,0x80,0x00,0x00,0x72,
		0x28,0x66,0x81,0x3e,0x70,0x05,0x00,0x80,0x01,0x00,0x77,
		0x1d,0x66,0x81,0x2e,0x74,0x05,0x00,0x80,0x00,0x00,0x66,
		0x81,0x2e,0x6c,0x05,0x00,0x80,0x00,0x00,0x66,0x81,0x2e,
		0x70,0x05,0x00,0x80,0x00,0x00,0xeb,0x0b,0x66,0x81,0x3e,
		0x70,0x05,0x00,0x00,0x01,0x00,0x77,0x2c,0x66,0xa1,0x6c,
		0x05,0x66,0x01,0x06,0x6c,0x05,0x66,0xa1,0x70,0x05,0x66,
		0x01,0x06,0x70,0x05,0x66,0xa1,0x74,0x05,0x66,0xd1,0xe0,
		0xe8,0x7d,0xfd,0x66,0x0f,0xb7,0x1e,0x9e,0x3f,0x66,0x03,
		0xc3,0x66,0xa3,0x74,0x05,0xe9,0x6e,0xff,0xff,0xb4,0x2f,
		0x18,0x8f,0x06,0x9e,0x3f,0xd1,0xee,0x56,0xe8,0xf3,0xfd,
		0x5e,0x66,0x61,0xc3,0x66,0x60,0x66,0x8b,0x1e,0x70,0x05,
		0x66,0x2b,0x1e,0x6c,0x05,0x66,0xa1,0x74,0x05,0x66,0x2b,
		0x06,0x6c,0x05,0x66,0x40,0x66,0x0f,0xb7,0x0e,0x91,0x1f,
		0x66,0xf7,0xe1,0x66,0x48,0x66,0x33,0xd2,0x66,0xf7,0xf3,
		0x50,0xe8,0x78,0xfe,0x58,0x8b,0x36,0x9e,0x3f,0xd1,0xe6,
		0x66,0x0f,0xb7,0x84,0x91,0x1f,0x66,0xf7,0xe3,0x66,0x33,
		0xd2,0x66,0x0f,0xb7,0x0e,0x91,0x1f,0x66,0xf7,0xf1,0x66,
		0x03,0x06,0x6c,0x05,0x66,0xa3,0x70,0x05,0x66,0x0f,0xb7,
		0x84,0x93,0x1f,0x66,0xf7,0xe3,0x66,0x33,0xd2,0x66,0xf7,
		0xf1,0x66,0x01,0x06,0x6c,0x05,0x66,0x81,0x3e,0x6c,0x05,
		0x00,0x00,0x01,0x00,0x72,0x1d,0x66,0x81,0x2e,0x74,0x05,
		0x00,0x00,0x01,0x00,0x66,0x81,0x2e,0x6c,0x05,0x00,0x00,
		0x01,0x00,0x66,0x81,0x2e,0x70,0x05,0x00,0x00,0x01,0x00,
		0xeb,0x3e,0x66,0x81,0x3e,0x6c,0x05,0x00,0x80,0x00,0x00,
		0x72,0x28,0x66,0x81,0x3e,0x70,0x05,0x00,0x80,0x01,0x00,
		0x77,0x1d,0x66,0x81,0x2e,0x74,0x05,0x00,0x80,0x00,0x00,
		0x66,0x81,0x2e,0x6c,0x05,0x00,0x80,0x00,0x00,0x66,0x81,
		0x2e,0x70,0x05,0x00,0x80,0x00,0x00,0xeb,0x0b,0x66,0x81,
		0x3e,0x70,0x05,0x00,0x00,0x01,0x00,0x77,0x2c,0x66,0xa1,
		0x6c,0x05,0x66,0x01,0x06,0x6c,0x05,0x66,0xa1,0x70,0x05,
		0x66,0x01,0x06,0x70,0x05,0x66,0xa1,0x74,0x05,0x66,0xd1,
		0xe0,0xe8,0x74,0xfc,0x66,0x0f,0xb7,0x1e,0x9e,0x3f,0x66,
		0x03,0xc3,0x66,0xa3,0x74,0x05,0xe9,0x6e,0xff,0xd1,0xee,
		0x89,0x36,0x9e,0x3f,0x66,0x61,0xc3,0x60,0x8b,0x0c,0x51,
		0x83,0xc6,0x04,0x89,0x36,0x7e,0x05,0xe8,0xc6,0xfd,0xe8,
		0x77,0xfc,0xb9,0xc4,0x0f,0xb0,0x20,0xbf,0x80,0x05,0x06,
		0x1e,0x07,0xf3,0xaa,0x07,0x5d,0xbb,0xc4,0x0f,0xe8,0xd1,
		0xfd,0xa1,0x9e,0x3f,0x3d,0x00,0x01,0x73,0x07,0xe8,0x3b,
		0x00,0x74,0x37,0xeb,0xee,0xe8,0xc8,0xfe,0x8b,0x16,0x9e,
		0x3f,0x8b,0xf3,0x2b,0xf2,0x4e,0x81,0xe6,0xff,0x0f,0x8b,
		0xf8,0x81,0xef,0xff,0x00,0x83,0xc7,0x02,0x8b,0xd7,0x33,
		0xc9,0x56,0x03,0xf1,0x81,0xe6,0xff,0x0f,0x8a,0x84,0x80,
		0x05,0xe8,0x0c,0x00,0x5e,0x74,0x07,0x41,0x3b,0xca,0x73,
		0xbb,0xeb,0xe8,0x61,0xc3,0x8b,0x3e,0x7c,0x05,0xaa,0xff,
		0x06,0x7c,0x05,0x88,0x87,0x80,0x05,0x43,0x81,0xe3,0xff,
		0x0f,0x4d,0xc3,0x00,0x30,0x80,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00
	    };


    if (argc == 1) 
    {
	printHelp();
	return 0;    
    }
    
    if (argc < 3) 
    {
	printf ("You have to specify input and output file\n");
	return 1;
    }

    
    //--------------------
    // parameter stuff
    //--------------------

    for (i2 = 1; i2 < argc - 2; i2++) 
    {
	paramok = 0;
	
	if (strcmp (argv[i2],"-h") == 0) 
	{
	    printHelp();
	    return 0;
	}
	
	if (strcmp (argv[i2],"-vendorid") == 0) 
	{
		if (argv[i2 + 1][1] == 'x') vendorid = tohex(argv[i2 + 1] + 2);
                else vendorid = atoi (argv[i2 + 1]);
		i2++;
		paramok = 1;
	
	}
	
	if (strcmp (argv[i2],"-deviceid") == 0) 
	{
		if (argv[i2 + 1][1] == 'x') deviceid = tohex(argv[i2 + 1] + 2);
                else deviceid = atoi (argv[i2 + 1]);
		i2++;
		paramok = 1;
	
	}
	
	if (strcmp (argv[i2],"-forceINT") == 0)
	{
	    forceINT = 1;
	    paramok = 1;
	}

	
	if (strcmp (argv[i2],"-INT18") == 0)
	{
	    INT18 = 1;
	    paramok = 1;
	}

	
	if (strcmp (argv[i2],"-hidden") == 0) 
	{
	    hidden = 1;
	    paramok = 1;
	}
	
	if (strcmp (argv[i2],"-nodisable") == 0) 
	{
	    disable = 0;
	    paramok = 1;
	}
	
	if (strcmp (argv[i2],"-compress") == 0) 
	{
	    encode = 1;
	    paramok = 1;
	}
	
	if (strcmp (argv[i2],"-grabid") == 0) 
	{
	    unsigned char tmpfilebuffer[4096];
	
	    paramok = 1;	    
	    if (!(pf = fopen (argv[i2 + 1], "rb"))) 
	    {
		printf ("Error: unable to open %s\n", argv[i2 + 1]);
		return 1;    
	    }

    
	    /*fseek (pf, 0, SEEK_END);
	    filesize = ftell (pf);
	    if (filesize > sizeof (tmpfilebuffer)) 
	    {
		printf ("Error: unable to read file, file too big\n");
		fclose (pf);
		return 1;        
	    }
	    */

	    fseek (pf, 0, SEEK_SET);
    
	    if (!fread (tmpfilebuffer, sizeof (tmpfilebuffer), 1, pf)) 
	    {
		printf ("Error: unable to read file %s\n", argv[i2 + 1]);
		fclose (pf);
		return 1;        
	    }

	    fclose (pf);
        
	    unsigned int ofs;
	    
	    ofs = (tmpfilebuffer[0x18] & 0xff)  + 
	        ((tmpfilebuffer[0x19] & 0xff) << 8 );
			     
	    ofs += 4;

	    vendorid = (tmpfilebuffer[ofs] & 0xff)  + 
		     ((tmpfilebuffer[ofs + 1] & 0xff) << 8 );
	    
	    deviceid = (tmpfilebuffer[ofs + 2] & 0xff)  + 
		     ((tmpfilebuffer[ofs + 3] & 0xff) << 8 );
	    
		    
	    printf ("Grabbing vendorid (0x%04x) and deviceid (0x%04x)\n",
		    vendorid, deviceid);

	    i2++;
	
	}
	
	if (strcmp (argv[i2],"-size") == 0) 
	{
	    paramok = 1;
	    
	    int factor = 1;
	    char val[1024];
	    
	    char *c = argv[i2 + 1] + strlen (argv[i2 + 1]) - 1;
	    strcpy (val, argv[i2 + 1]);

	    switch (c[0])
	    {
		case 'k':
		    factor = 1024;
		    val[strlen (val) - 1] = 0;
		    break;
	    
		case 'm':
		    factor = 1024 * 1024;
		    val[strlen (val) - 1] = 0;
		    break;	    
	    }
	    
	    size = setsize = atoi (val) * factor;
	    if (setsize == 0)
	    {
		printf ("Size error\n");
		return 1;
	    }
	    printf ("Set size to %s (%d bytes)\n", argv[i2 + 1], setsize);
	    i2++;
	}
	
	if (!paramok)
	{
	    printf ("Unknown parameter %s\n", argv[i2]);
	    return 1;
	
	}
    }


    strcpy (inputfile,argv[argc - 2]);
    strcpy (outputfile,argv[argc - 1]);


    unsigned char filebuffer[size];
    unsigned char outputbuffer[size];
    unsigned char compressedbuffer[size];
    

    //--------------------
    // read file
    //--------------------
    if (!(pf = fopen (inputfile, "rb"))) 
    {
	printf ("Error: unable to open %s\n", inputfile);
	return 1;    
    }
    
    fseek (pf, 0, SEEK_END);
    filesize = ftell (pf);
    
    if (filesize > size) 
    {
	printf ("Error: unable to read file, file too big\n");
	fclose (pf);
	return 1;        
    }
    

    fseek (pf, 0, SEEK_SET);
    
    if (!fread (filebuffer, filesize, 1, pf)) 
    {
	printf ("Error: unable to read file\n");
	fclose (pf);
	return 1;        
    }
    
    fclose (pf);
    




    if (encode)
	cfilesize = Encode (filebuffer, filesize, compressedbuffer);


    // generate output file

    for (i = 0; i < size; i++)
	outputbuffer[i] = 0x00;


	if (encode)
	    fullsize = (512 - ((cfilesize + sizeof (pciheader) +
	    		      sizeof (decoder)) % 512)) +
		     cfilesize + sizeof (pciheader) + sizeof (decoder);
	else
	    fullsize = (512 - ((filesize + sizeof (pciheader)) % 512)) +
	    	     filesize + sizeof (pciheader);
	
	if (setsize)
	    fullsize = setsize;


	// copy pci header
	for (i = 0; i < sizeof (pciheader); i++)
	    outputbuffer[i] = pciheader[i];


	// copy boot manager
	if (encode)
	{
	    for (i = 0; i < sizeof (decoder); i++)
		outputbuffer[i + sizeof (pciheader)] = decoder[i];

	    for (i = 0; i < cfilesize; i++)
		outputbuffer[i + sizeof (pciheader) 
			       + sizeof (decoder)] = compressedbuffer[i];
	}
	else
	    for (i = 0; i < filesize; i++)
		outputbuffer[i + sizeof (pciheader)] = filebuffer[i];

	// rom size in 512 byte blocks
	outputbuffer[0x02] = fullsize >> 9;
	outputbuffer[0x2c] = fullsize >> 9;
	
	if (vendorid) 
	{
	    outputbuffer[0x20] = vendorid & 0xff;
	    outputbuffer[0x21] = (vendorid >> 8) & 0xff;
	}

	if (deviceid) 
	{
	    outputbuffer[0x22] = deviceid & 0xff;
	    outputbuffer[0x23] = (deviceid >> 8) & 0xff;
	}

	outputbuffer[0x55] = forceINT;
	outputbuffer[0x56] = INT18;
	outputbuffer[0x57] = hidden;
	outputbuffer[0x58] = disable;

	// calc checksum
    	for (i = 0; i < fullsize; i++)
	    chksum += outputbuffer[i] &0xff;
	
	chksum = 0x100 - chksum;

//	outputbuffer[0x06]=chksum;
	outputbuffer[13] = chksum;




	if (!(pf = fopen (outputfile, "wb"))) 
	{
	    printf ("Error: unable to create %s\n", outputfile);
	    return 1;
	}
	
	fwrite (outputbuffer, fullsize, 1, pf);
	fclose (pf);
	
	
	    


    printf ("%s created\n", outputfile);
    return 0;
}

